import pygtk
pygtk.require('2.0')
import gtk
import pango
import dbus
import cgi
import xml.sax.saxutils
import re
import gobject

import google
from sugar.presence.PresenceService import PresenceService
from sugar.presence import Service
from sugar.browser import BrowserActivity

_COLUMN_TITLE = 0
_COLUMN_ADDRESS = 1
_COLUMN_SUBTITLE = 2
_COLUMN_SERVICE = 3

class SearchHelper(object):
	def __init__(self, activity_uid):
		self.search_uid = activity_uid
		self.found = False

class ActivitiesModel(gtk.ListStore):
	def __init__(self):
		gtk.ListStore.__init__(self, gobject.TYPE_STRING, gobject.TYPE_STRING,
				gobject.TYPE_STRING, gobject.TYPE_PYOBJECT)

	def add_web_page(self, title, address):
		self.append([ title, address, None, None ])

	def _filter_dupe_activities(self, model, path, it, user_data):
		"""Search the list of list rows for an existing service that
		has the activity ID we're looking for."""
		helper = user_data
		(service, ) = model.get(it, _COLUMN_SERVICE)
		if not service:
			return False
		if service.get_activity_uid() == helper.search_uid:
			helper.found = True
			return True
		return False
	
	def add_activity(self, buddy, service):
		# Web Activity check
		activity_uid = service.get_activity_uid()
		if activity_uid is None:
			return
		# Don't show dupes
		helper = SearchHelper(activity_uid)
		self.foreach(self._filter_dupe_activities, helper)
		if helper.found == True:
			return

		# Only accept browser activities for now
		if service.get_type() == BrowserActivity._BROWSER_ACTIVITY_TYPE:
			escaped_title = service.get_one_property('Title')
			escaped_uri = service.get_one_property('URI')
			if escaped_title and escaped_uri:
				title = xml.sax.saxutils.unescape(escaped_title)
				address = xml.sax.saxutils.unescape(escaped_uri)
				subtitle = 'Shared by %s' % buddy.get_nick_name()
				self.append([ title, address, subtitle, service ])

class ActivitiesView(gtk.TreeView):
	def __init__(self):
		gtk.TreeView.__init__(self, model = ActivitiesModel())
		
		self.set_headers_visible(False)
		
		column = gtk.TreeViewColumn('')
		self.append_column(column)

		cell = gtk.CellRendererText()
		column.pack_start(cell, True)
		column.set_cell_data_func(cell, self._cell_data_func)
		
		self.connect('row-activated', self._row_activated_cb)
	
	def _cell_data_func(self, column, cell, model, it):
		title = model.get_value(it, _COLUMN_TITLE)
		subtitle = model.get_value(it, _COLUMN_SUBTITLE)
		if subtitle is None:
			subtitle = model.get_value(it, _COLUMN_ADDRESS)

		markup = '<big><b>' + cgi.escape(title) + '</b></big>' 
		markup += '\n' + cgi.escape(subtitle)
		
		cell.set_property('markup', markup)
		cell.set_property('ellipsize', pango.ELLIPSIZE_END)
			
	def _row_activated_cb(self, treeview, path, column):	
		bus = dbus.SessionBus()
		proxy_obj = bus.get_object('com.redhat.Sugar.Browser', '/com/redhat/Sugar/Browser')
		browser_shell = dbus.Interface(proxy_obj, 'com.redhat.Sugar.BrowserShell')

		model = self.get_model() 
		address = model.get_value(model.get_iter(path), _COLUMN_ADDRESS)
		service = model.get_value(model.get_iter(path), _COLUMN_SERVICE)

		print 'Activated row %s' % address

		if service is None:
			browser_shell.open_browser(address)
		else:
			serialized_service = service.serialize()
			browser_shell.open_browser(address, serialized_service)
			
class StartPage(gtk.HBox):
	def __init__(self, ac_signal_object):
		gtk.HBox.__init__(self)

		self._ac_signal_object = ac_signal_object
		self._ac_signal_object.connect("local-activity-started",
				self._on_local_activity_started_cb)
		self._ac_signal_object.connect("local-activity-ended",
				self._on_local_activity_ended_cb)

		self._pservice = PresenceService.get_instance()
		self._pservice.connect("activity-announced", self._on_activity_announced_cb)
		self._pservice.connect("new-service-adv", self._on_new_service_adv_cb)
		self._pservice.start()
		self._pservice.track_service_type(BrowserActivity._BROWSER_ACTIVITY_TYPE)

		vbox = gtk.VBox()

		search_box = gtk.HBox(False, 6)
		search_box.set_border_width(24)
		
		self._search_entry = gtk.Entry()
		self._search_entry.connect('activate', self._search_entry_activate_cb)
		search_box.pack_start(self._search_entry)
		self._search_entry.show()
		
		search_button = gtk.Button("Search")
		search_button.connect('clicked', self._search_button_clicked_cb)
		search_box.pack_start(search_button, False)
		search_button.show()

		vbox.pack_start(search_box, False, True)
		search_box.show()

		exp_space = gtk.Label('')
		vbox.pack_start(exp_space)
		exp_space.show()
				
		self.pack_start(vbox)
		vbox.show()

		sw = gtk.ScrolledWindow()
		sw.set_size_request(320, -1)
		sw.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)

		self._activities = ActivitiesView()
		sw.add(self._activities)
		self._activities.show()

		self.pack_start(sw)	
		sw.show()

	def _on_local_activity_started_cb(self, helper, activity_container, activity_id):
		self._pservice.track_activity(activity_id)
		print "new local activity %s" % activity_id

	def _on_local_activity_ended_cb(self, helper, activity_container, activity_id):
		self._pservice.untrack_activity(activity_id)

	def _on_new_service_adv_cb(self, pservice, activity_id, short_stype):
		if activity_id:
			self._pservice.track_activity(activity_id)
			self._pservice.track_service_type(short_stype)

	def _on_activity_announced_cb(self, pservice, service, buddy):
		print "Found new activity with type %s" % service.get_full_type()
		self._activities.get_model().add_activity(buddy, service)

	def _search_entry_activate_cb(self, entry):
		self._search()
		
	def _search_button_clicked_cb(self, button):
		self._search()
	
	def _search(self):
		text = self._search_entry.get_text()
		self._search_entry.set_text('')
	
		google.LICENSE_KEY = '1As9KaJQFHIJ1L0W5EZPl6vBOFvh/Vaf'
		data = google.doGoogleSearch(text)
		
		model = ActivitiesModel()
		for result in data.results:
			title = result.title
			
			# FIXME what tags should we actually strip?
			title = title.replace('<b>', '') 
			title = title.replace('</b>', '')

			# FIXME I'm sure there is a better way to
			# unescape these.
			title = title.replace('&quot;', '"')
			title = title.replace('&amp;', '&')

			model.add_web_page(title, result.URL)
		self._activities.set_model(model)
